use std::{future::Future, pin::Pin, sync::Arc, time::Duration};

use chrono_tz::Tz;
use tokio_cron_scheduler::{Job, JobScheduler, JobSchedulerError};

type TaskFun = dyn Fn() -> Pin<Box<dyn Future<Output = ()> + Send>> + Send + Sync;

pub struct Task{
    pub job: Arc<TaskFun>,
    pub cron: Option<String>,
    pub interval: Option<u64>,
    pub time_zone: Option<String>,
}

/// 启动定时任务
pub async fn start(tasks: Vec<Task>) {
    match schedule(tasks).await {
        Ok(_) => tracing::info!("定时任务启动成功"),
        Err(err) => tracing::error!(error = ?err, "定时任务启动失败"),
    }
}

async fn schedule(tasks: Vec<Task>) -> Result<(), JobSchedulerError> {
    let mut scheduler = JobScheduler::new().await?;
    scheduler.init().await?;

    // 添加任务
    for task in tasks {
        let task: &'static Task = Box::leak(Box::new(task));
        if task.cron.is_some() {
            let schedule = task.cron.as_ref().unwrap().as_str();
            if let Some(time_zone) = task.time_zone.as_ref() {
                tracing::info!("添加定时任务: {} - {}", task.cron.as_ref().unwrap(), time_zone);
                let tz: Tz = time_zone.parse().unwrap();
                let job_fun = task.job.clone();
                let _task_uuid = scheduler.add(Job::new_cron_job_async_tz(schedule, tz, move |_uuid, _l| {
                    job_fun()
                })?).await?;
            } else {
                tracing::info!("添加定时任务: {}", task.cron.as_ref().unwrap());
                let job_fun = task.job.clone();
                let _task_uuid = scheduler.add(Job::new_cron_job_async(schedule, move |_uuid, _l| {
                    job_fun()
                })?).await?;
            }
        } else if task.interval.is_some() {
            let schedule = task.interval.unwrap();
            tracing::info!("添加定时任务: {}", task.interval.as_ref().unwrap());
            let job_fun = task.job.clone();
            let _task_uuid = scheduler.add(Job::new_repeated_async(Duration::from_secs(schedule), move |_uuid, _l| {
                job_fun()
            })?).await?;
        }
    }

    // 添加关闭监听
    scheduler.set_shutdown_handler(Box::new(|| {
        Box::pin(async move {
            tracing::info!("定时任务已关闭");
        })
    }));

    // 启动任务
    scheduler.start().await?;
    Ok(())
}
